﻿/*
____
 L|ypps Web controls for ASP.NET

Copyright (c) 2009 Alessandro Zifiglio, http://www.Typps.com
EMail: alessandro@Typps.com

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
namespace Typps
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.Design;
    using System.Drawing.Design;
    using System.Runtime.Serialization;
    using System.Text;
    using System.Web.UI;
    using System.Web.UI.WebControls;

    [CollectionDataContract,
    KnownType(typeof(TabItem)),
    Editor(typeof(Typps.ToolBarCollectionEditor), typeof(UITypeEditor))]
    public class TabCollection : StateManagedCollection
    {
        #region Fields

        private static readonly Type[] knownTypes;

        #endregion Fields

        #region Constructors

        static TabCollection()
        {
            TabCollection.knownTypes = new Type[] { typeof(TabItem) };
        }

        public TabCollection()
        {
        }

        #endregion Constructors

        #region Indexers

        public TabItem this[int index]
        {
            get
            {
                return (TabItem)((IList)this)[index];
            }
            set
            {
                ((IList)this)[index] = value;
            }
        }

        #endregion Indexers

        #region Methods

        // Restricts to TabItem types, items that can be added to the collection
        public int Add(TabItem value)
        {
            return ((IList)this).Add(value);
        }

        public bool Contains(TabItem value)
        {
            // If value is not of type TabItem, this will return false.
            return ((IList)this).Contains(value);
        }

        public void CopyTo(TabItem[] toolBarSetArray, int index)
        {
            base.CopyTo(toolBarSetArray, index);
        }

        public int IndexOf(TabItem value)
        {
            return ((IList)this).IndexOf(value);
        }

        public void Insert(int index, TabItem value)
        {
            ((IList)this).Insert(index, value);
        }

        public void Remove(TabItem value)
        {
            ((IList)this).Remove(value);
        }

        public void RemoveAt(int index)
        {
            ((IList)this).RemoveAt(index);
        }

        protected override object CreateKnownType(int index)
        {
            return new TabItem();
        }

        protected override Type[] GetKnownTypes()
        {
            return TabCollection.knownTypes;
        }

        protected override void OnClear()
        {
            base.OnClear();
        }

        protected override void OnInsert(int index, object value)
        {
            base.OnInsert(index, value);
            if (!(value is TabItem))
            {
                throw new ArgumentException("value must be of type TabItem.", "value");
            }
        }

        protected override void OnValidate(Object value)
        {
            if (value.GetType() != typeof(TabItem))
                throw new ArgumentException("value must be of type TabItem.", "value");
        }

        protected override void SetDirtyObject(object o)
        {
            if (o is TabItem)
            {
                ((TabItem)o).SetDirty();
            }
        }

        #endregion Methods
    }
}