﻿/*
____
 L|ypps Web controls for ASP.NET

Copyright (c) 2009 Alessandro Zifiglio, http://www.Typps.com
EMail: alessandro@Typps.com

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
namespace Typps
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.Design;
    using System.Drawing.Design;
    using System.Runtime.Serialization;
    using System.Text;
    using System.Web.UI;

    [CollectionDataContract]
    public class ToolBarItemCollection : StateManagedCollection
    {
        #region Fields

        private static readonly Type[] knownTypes;

        #endregion Fields

        #region Constructors

        static ToolBarItemCollection()
        {
            ToolBarItemCollection.knownTypes = new Type[] { typeof(ToolBarItem) };
        }

        public ToolBarItemCollection()
        {
        }

        #endregion Constructors

        #region Indexers

        public ToolBarItem this[int index]
        {
            get
            {
                return (ToolBarItem)((IList)this)[index];
            }
            set
            {
                ((IList)this)[index] = value;
            }
        }

        #endregion Indexers

        #region Methods

        // Restricts to ToolBarItem types, items that can be added to the collection
        public int Add(ToolBarItem value)
        {
            return ((IList)this).Add(value);
        }

        public bool Contains(ToolBarItem value)
        {
            // If value is not of type ToolBarItem, this will return false.
            return ((IList)this).Contains(value);
        }

        public void CopyTo(ToolBarItem[] toolBarItemArray, int index)
        {
            base.CopyTo(toolBarItemArray, index);
        }

        public int IndexOf(ToolBarItem value)
        {
            return ((IList)this).IndexOf(value);
        }

        public void Insert(int index, ToolBarItem value)
        {
            ((IList)this).Insert(index, value);
        }

        public void Remove(ToolBarItem value)
        {
            ((IList)this).Remove(value);
        }

        protected override object CreateKnownType(int index)
        {
            return new ToolBarItem();
        }

        protected override Type[] GetKnownTypes()
        {
            return ToolBarItemCollection.knownTypes;
        }

        protected override void OnClear()
        {
            base.OnClear();
        }

        protected override void OnInsert(int index, object value)
        {
            base.OnInsert(index, value);
            if (!(value is ToolBarItem))
            {
                throw new ArgumentException("value must be of type ToolBarItem.", "value");
            }
        }

        protected override void OnValidate(Object value)
        {
            if (value.GetType() != typeof(ToolBarItem))
                throw new ArgumentException("value must be of type ToolBarItem.", "value");
        }

        protected override void SetDirtyObject(object o)
        {
            if (o is ToolBarItem)
            {
                ((ToolBarItem)o).SetDirty();
            }
        }

        #endregion Methods
    }
}