﻿/*
____
 L|ypps Web controls for ASP.NET

Copyright (c) 2009 Alessandro Zifiglio, http://www.Typps.com
EMail: alessandro@Typps.com

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
namespace Typps
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.Design;
    using System.Drawing.Design;
    using System.Runtime.Serialization;
    using System.Text;
    using System.Web.Script.Serialization;
    using System.Web.UI;

    [DataContract]
    public class DictionaryItem : IStateManager
    {
        #region Fields

        bool defaultValue = false;
        private bool isTrackingViewStateValue;
        private StateBag viewStateValue;

        #endregion Fields

        #region Constructors

        public DictionaryItem()
            : this(string.Empty, string.Empty, string.Empty, false)
        {
            // overloaded constructor
        }

        public DictionaryItem(string t, string v, string p, bool sel)
        {
            this.Text = t;
            this.Value = v;
            this.Path = p;
            this.Selected = sel;
        }

        #endregion Constructors

        #region Properties

        [IgnoreDataMember,
        Bindable(false),
        Category("Behavior"),
        Editor(typeof(System.Web.UI.Design.ImageUrlEditor), typeof(UITypeEditor)),
        DefaultValue(""),
        NotifyParentProperty(true),
        Description("Gets or Sets the path to the .dic file")]
        public string Path
        {
            get
            {
                object o = (object)ViewState["Path"];
                return (o == null) ? string.Empty : o.ToString();
            }
            set
            {
                defaultValue = false;
                ViewState["Path"] = value;
            }
        }

        [DataMember(Name="selected"),
        Bindable(false),
        Category("Behavior"),
        DefaultValue(false),
        NotifyParentProperty(true),
        Description("Gets or sets a value indicating whether the item is selected.")]
        public bool Selected
        {
            get
            {
                object o = (object)ViewState["Selected"];
                return (o == null) ? false : (bool)o;
            }
            set
            {
                defaultValue = false;
                ViewState["Selected"] = value;
            }
        }

        [DataMember(Name="text"),
        Bindable(false),
        Category("Behavior"),
        DefaultValue(""),
        NotifyParentProperty(true),
        Description("Gets or sets the text displayed in the dictionary list represented by this DictionaryItem")]
        public string Text
        {
            get
            {
                object o = (object)ViewState["Text"];
                return (o == null) ? string.Empty : (string)o;
            }
            set
            {
                defaultValue = false;
                ViewState["Text"] = value;
            }
        }

        [DataMember(Name="value"),
        Bindable(false),
        Category("Behavior"),
        DefaultValue(""),
        NotifyParentProperty(true),
        Description("Gets or sets the value associated with the DictionaryItem")]
        public string Value
        {
            get
            {
                object o = (object)ViewState["Value"];
                return (o == null) ? string.Empty : o.ToString();
            }
            set
            {
                defaultValue = false;
                ViewState["Value"] = value;
            }
        }

        bool IStateManager.IsTrackingViewState
        {
            get
            {
                return isTrackingViewStateValue;
            }
        }

        internal bool Defaults
        {
            get
            {
                return defaultValue;
            }
            set
            {
                defaultValue = value;
            }
        }

        private StateBag ViewState
        {
            get
            {
                if (viewStateValue == null)
                {
                    viewStateValue = new StateBag(false);
                    if (isTrackingViewStateValue)
                    {
                        ((IStateManager)viewStateValue).TrackViewState();
                    }
                }
                return viewStateValue;
            }
        }

        #endregion Properties

        #region Methods

        void IStateManager.LoadViewState(object savedState)
        {
            this.LoadViewState(savedState);
        }

        object IStateManager.SaveViewState()
        {
            if (Defaults)
                return null;

            return this.SaveViewState();
        }

        void IStateManager.TrackViewState()
        {
            isTrackingViewStateValue = true;
            if (viewStateValue != null)
                ((IStateManager)viewStateValue).TrackViewState();
        }

        internal void LoadViewState(object savedState)
        {
            object[] states = (object[])savedState;
            ((IStateManager)ViewState).LoadViewState(states[0]);
        }

        internal object SaveViewState()
        {
            object[] states = new object[1];
            states[0] = (viewStateValue != null) ? ((IStateManager)viewStateValue).SaveViewState() : null;
            return states;
        }

        internal void SetDirty()
        {
            viewStateValue.SetDirty(true);
        }

        #endregion Methods
    }
}