﻿namespace Typps
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Linq;
    using System.Runtime.Serialization;
    using System.Text;
    using System.Web.UI;
    using System.Web.UI.WebControls;

    [DataContract]
    public class Shake : BaseEffect, IStateManager
    {
        #region Fields

        double durationValue;
        private bool isTrackingViewStateValue;
        private StateBag viewStateValue;

        #endregion Fields

        #region Constructors

        public Shake()
        {
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Fixed at 500 milliseconds. You cannot set this value.
        /// </summary>
        [DataMember(Name="duration"),
        DefaultValue(500)]
        public double Duration
        {
            get { return 500d; }
            set
            {
                durationValue = value;
            }
        }

        /// <summary>
        /// Lets you define a function that is called after the effect 
        /// finishes, e.g., function (element,effect){/* ... */}.
        /// </summary>
        [DefaultValue(null),
        Browsable(false)]
        public override string Finish
        {
            get { return (string)(ViewState["Finish"] ?? null); }
            set { ViewState["Finish"] = value; }
        }

        /// <summary>
        /// Determines the number of frames per second of the animation. The default is 60.
        /// </summary>
        [DataMember(Name="fps"),
        DefaultValue(60)]
        public int FPS
        {
            get { return (int)(ViewState["FPS"]??60); }
            set{ViewState["FPS"] = value;}
        }

        public override string Name
        {
            get { return "Shake"; }
            internal set { }
        }

        /// <summary>
        /// Using AddParallelEffect causes all of the registered effects to run in parallel.
        /// In other words, the registered Effects run together.
        /// While using AddNextEffect causes all of the registered effects to run one after the other.
        /// In other words, as one effects finishes, the next registered effect will run and so on. 
        /// The default is AddParellelEffect (when using UseDefault dictated by the control using this effect). When having registered more than
        /// one effect, RunMode set on this individual Effect has priority. This means if you set the 
        /// runtime mode explicity other than UseDefault, it will have precedence over runtime mode set directly on
        /// the control. Set runtime on the control instead of here, if all your effects share the same
        /// runtime mode. 
        /// </summary>
        public override EffectRunMode RunMode
        {
            get { return (EffectRunMode)(ViewState["RunMode"] ?? EffectRunMode.UseDefault); }
            set { ViewState["RunMode"] = value; }
        }

        /// <summary>
        /// Lets you define a function that is called before the effect 
        /// begins, e.g., function (element,effect){/* ... */}.
        /// </summary>
        [DefaultValue(null),
        Browsable(false)]
        public override string Setup
        {
            get { return (string)(ViewState["Setup"] ?? null); }
            set { ViewState["Setup"] = value; }
        }

        /// <summary>
        /// Determines the type of transition. The default is 'squareTransition'.
        /// </summary>
        [DataMember(Name="transition"),
        DefaultValue(TransitionName.linearTransition)]
        public TransitionName Transition
        {
            get { return (TransitionName)(ViewState["Transition"]??TransitionName.linearTransition); }
            set { ViewState["Transition"] = value; }
        }

        bool IStateManager.IsTrackingViewState
        {
            get
            {
                return isTrackingViewStateValue;
            }
        }

        private StateBag ViewState
        {
            get
            {
                if (viewStateValue == null)
                {
                    viewStateValue = new StateBag(false);
                    if (isTrackingViewStateValue)
                    {
                        ((IStateManager)viewStateValue).TrackViewState();
                    }
                }
                return viewStateValue;
            }
        }

        #endregion Properties

        #region Methods

        void IStateManager.LoadViewState(object savedState)
        {
            this.LoadViewState(savedState);
        }

        object IStateManager.SaveViewState()
        {
            return this.SaveViewState();
        }

        void IStateManager.TrackViewState()
        {
            isTrackingViewStateValue = true;
            if (viewStateValue != null)
                ((IStateManager)viewStateValue).TrackViewState();
        }

        public override void SetDirty()
        {
            if (viewStateValue != null)
                viewStateValue.SetDirty(true);
        }

        internal void LoadViewState(object savedState)
        {
            object[] states = (object[])savedState;
            ((IStateManager)ViewState).LoadViewState(states[0]);
        }

        internal object SaveViewState()
        {
            object[] states = new object[1];
            states[0] = (viewStateValue != null) ? ((IStateManager)viewStateValue).SaveViewState() : null;
            return states;
        }

        #endregion Methods
    }
}