﻿/*
____
 L|ypps Web controls for ASP.NET

Copyright (c) 2009 Alessandro Zifiglio, http://www.Typps.com
EMail: alessandro@Typps.com

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
namespace Typps
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.Design;
    using System.Drawing;
    using System.Drawing.Design;
    using System.Runtime.Serialization;
    using System.Text;
    using System.Web.UI;
    using System.Web.UI.WebControls;

    [DataContract(Name="tabItem",Namespace="")]
    public class TabItem : IStateManager
    {
        #region Fields

        bool defaultValue = false;
        private bool isTrackingViewStateValue;
        private StateBag viewStateValue;

        #endregion Fields

        #region Constructors

        public TabItem()
            : this(0, string.Empty, string.Empty, Unit.Empty)
        {
            // overloaded constructor
        }

        public TabItem(int ml, string c, string tt, Unit w)
        {
            this.MarginLeft = ml;
            this.Command = c;
            this.ToolTip = tt;
            this.Width = w;
        }

        #endregion Constructors

        #region Properties

        [DataMember(Name="divider"),
        Category("Behavior"),
        DefaultValue(false),
        NotifyParentProperty(true),
        Description("Gets or Sets a divider after this item")]
        public bool Break
        {
            get
            {
                object o = (object)ViewState["Break"];
                return (o == null) ? false : (bool)o;
            }
            set
            {
                defaultValue = false;
                ViewState["Break"] = value;
            }
        }

        [DataMember(Name="command"),
        Bindable(false),
        Category("Behavior"),
        DefaultValue(""),
        NotifyParentProperty(true),
        Description("Gets or Sets the command to call on execCommand method exposed through the dom")]
        public string Command
        {
            get
            {
                object o = (object)ViewState["Command"];
                return (o == null) ? string.Empty : o.ToString();
            }
            set
            {
                defaultValue = false;
                ViewState["Command"] = value;
            }
        }

        [DataMember(Name="enabled"),
        Category("Behavior"),
        DefaultValue(true),
        NotifyParentProperty(true),
        Description("Gets or Sets if this icon is Enabled")]
        public bool Enabled
        {
            get
            {
                object o = (object)ViewState["Enabled"];
                return (o == null) ? true : (bool)o;
            }
            set
            {
                defaultValue = false;
                ViewState["Enabled"] = value;
            }
        }

        [DataMember(Name="icon"),
        Editor(typeof(System.Web.UI.Design.ImageUrlEditor), typeof(UITypeEditor)),
        DefaultValue(""),
        NotifyParentProperty(true)]
        public string Icon
        {
            get
            {
                return (ViewState["Icon"] == null) ? string.Empty : (string)ViewState["Icon"];
            }
            set
            {
                defaultValue = false;
                ViewState["Icon"] = value;
            }
        }

        [DataMember(Name="marginLeft"),
        Bindable(false),
        Category("Behavior"),
        DefaultValue(0),
        NotifyParentProperty(true),
        Description("Left margin spacing of your icon with respect to left corner of tab, if its the first icon or right corner of the icon before it.")]
        public int MarginLeft
        {
            get
            {
                object o = (object)ViewState["MarginLeft"];
                return (o == null) ? 0 : (int)o;
            }
            set
            {
                defaultValue = false;
                ViewState["MarginLeft"] = value;
            }
        }

        [DataMember(Name="title"),
        Bindable(false),
        Category("Behavior"),
        DefaultValue(""),
        NotifyParentProperty(true),
        Description("Gets or Sets the tooltip for this icon")]
        public string ToolTip
        {
            get
            {
                object o = (object)ViewState["ToolTip"];
                return (o == null) ? string.Empty : o.ToString();
            }
            set
            {
                defaultValue = false;
                ViewState["ToolTip"] = value;
            }
        }

        [DataMember(Name="width"),
        Bindable(false),
        Category("Behavior"),
        DefaultValue(0),
        NotifyParentProperty(true),
        Description("The width of this tab item.")]
        public Unit Width
        {
            get
            {
                object o = (object)ViewState["Width"];
                return (o == null) ? Unit.Empty : (Unit)o;
            }
            set
            {
                defaultValue = false;
                ViewState["Width"] = value;
            }
        }

        bool IStateManager.IsTrackingViewState
        {
            get
            {
                return isTrackingViewStateValue;
            }
        }

        internal bool Defaults
        {
            get
            {
                return defaultValue;
            }
            set
            {
                defaultValue = value;
            }
        }

        private StateBag ViewState
        {
            get
            {
                if (viewStateValue == null)
                {
                    viewStateValue = new StateBag(false);
                    if (isTrackingViewStateValue)
                    {
                        ((IStateManager)viewStateValue).TrackViewState();
                    }
                }
                return viewStateValue;
            }
        }

        #endregion Properties

        #region Methods

        void IStateManager.LoadViewState(object savedState)
        {
            this.LoadViewState(savedState);
        }

        object IStateManager.SaveViewState()
        {
            if (Defaults)
                return null;
            return this.SaveViewState();
        }

        void IStateManager.TrackViewState()
        {
            isTrackingViewStateValue = true;
            if (viewStateValue != null)
                ((IStateManager)viewStateValue).TrackViewState();
        }

        internal void LoadViewState(object savedState)
        {
            object[] states = (object[])savedState;
            if (states != null)
            {
                ((IStateManager)ViewState).LoadViewState(states[0]);
            }
        }

        internal object SaveViewState()
        {
            object[] states = new object[1];
            states[0] = (viewStateValue != null) ? ((IStateManager)viewStateValue).SaveViewState() : null;
            return states;
        }

        internal void SetDirty()
        {
            viewStateValue.SetDirty(true);
        }

        #endregion Methods
    }
}