﻿/*
____
 L|ypps Web controls for ASP.NET

Copyright (c) 2009 Alessandro Zifiglio, http://www.Typps.com
EMail: alessandro@Typps.com

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
namespace Typps
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.Design;
    using System.Configuration;
    using System.Drawing;
    using System.IO;
    using System.Text;
    using System.Web;
    using System.Web.Configuration;
    using System.Web.UI;
    using System.Web.UI.Design;
    using System.Web.UI.Design.WebControls;
    using System.Web.UI.WebControls;
    using System.Windows.Forms;
    using System.Xml;

    public class ColorPickerDesigner : BaseControlDesigner
    {
        #region Fields

        string colorPickerCss = string.Empty;

        //private DesignerAutoFormatCollection autoFormatsValue = null;
        FormCss form1 = null;
        string[] styles = null;

        #endregion Fields

        #region Methods

        public override string GetDesignTimeHtml()
        {
            ColorPicker baseComponent = (ColorPicker)ViewControl;
            string designTimeHtml = base.GetDesignTimeHtml();
            if (baseComponent.TargetControl != string.Empty)
            {
                designTimeHtml = string.Format("<span style=\"border:1px outset #000;background:Gainsboro;font-size:12pt;font-family:Calibri,'Courier New',Arial;color:black\">TargetControl[{0}]</span>", baseComponent.TargetControl);
            }
            else
            {
                string css = string.Format("<style type=\"text/css\">{0}</style>", baseComponent.GetCssStyle());
                designTimeHtml = string.Format("{0}{1}", css, designTimeHtml);
            }
            return designTimeHtml;
        }

        protected override void OnRetrieveAutoGeneratedCss(object sender, EventArgs args)
        {
            if (form1 == null)
                form1 = new FormCss(CssSelection.ColorPicker);

            ColorPicker baseComponent = (ColorPicker)ViewControl;

            string colorPickerCss = baseComponent.GetCssStyle(true);
            string updatePanelProgressCss = baseComponent.EmbeddedUpdatePanelProgress.GetCssStyle(true);
            string sharedCss = baseComponent.GetSharedCssStyle();

            form1.CssSelectedIndexChanged += new CssSelectedIndexChangedEventHandler(form1_CssSelectedIndexChanged);
            form1.WhiteSpaceCheckedChanged += new WhiteSpaceCheckedChangedEventHandler(form1_WhiteSpaceCheckedChanged);

            styles = new string[]
            {
                colorPickerCss,
                updatePanelProgressCss,
                sharedCss
            };
            form1.SetCss(string.Join("\r", styles));

            form1.ShowDialog();
        }

        protected override void PreFilterProperties(System.Collections.IDictionary properties)
        {
            base.PreFilterProperties(properties);
        }

        void form1_CssSelectedIndexChanged(object sender, CssSelectedIndexChangedEventArgs e)
        {
            SetCss(e.SelectedIndex, form1.IsWhiteSpaceRemovalChecked());
        }

        void form1_WhiteSpaceCheckedChanged(object sender, WhiteSpaceCheckedChangedEventArgs e)
        {
            SetCss(e.SelectedIndex, e.CheckedStatus);
        }

        void SetCss(CssSelection cs, bool removeWhiteSpace)
        {
            string css = string.Empty;
            switch (cs)
            {
                case CssSelection.ColorPicker:
                    css = string.Join("\r", styles);
                    break;
            }
            if (removeWhiteSpace && css != string.Empty)
                css = Helper.TrimWhiteSpace(css);
            form1.SetCss(css);
        }

        #endregion Methods
    }
}